function ExpData = processall(idFile)
    % PROCESSALL Process all data files listed in idFile.
    %   This function is the main processing tool for all the experimental data related to the
    %   MechaRaptor.
    %
    %   It parses the idFile, then opens, cleans, filter and synchronize the data from the wind
    %   tunnel and the arduino so they can be used for further analysis.
    % -----
    %
    % Syntax:
    %   processall(idFile) process all raw data files found in idFile.
    %
    % Inputs:
    %   idFile: Identification file.
    %
    % See also: readdata.

    % ----------------------------------------------------------------------------------------------
    % TODO:
    %   1. Load identification file
    %   2. Load Arduino file into table
    %       - cleanup table from unneeded columns (keep raw angles, volt, curr)
    %       - re-align angles properly from raw position
    %       - output table has proper angles, powers
    %   3. Load wind tunnel data
    %       - low pass filter, based on id file (filter > 1.5 x freq)
    %   4. Sync data and interpolate
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    close all;
    clc;

    addpath(genpath('.'));

    % Defaults and constants
    PLOT_CLEANED = false;
    CUTOFF_MULTIPLIER = 1.5; % Multiplier for the theorical frequency to use as a cutoff, [-]

    % Input checks and processing
    % [filePaths, fileNames] = getfiles(idFile); %TODO

    % FIXME: Detect automatically from idFile
    fileRootDir = 'data/2022-campaign1'; % Root directory of data files (e.g. data/YYYY-campaignX)

    idTable = loadid(idFile);

    ExpData(size(idTable, 1)) = struct;
    for iTest = 1:size(idTable, 1)

        ExpData(iTest).arduFile = fullfile(fileRootDir, 'arduinoData', ...
                                           idTable(iTest, :).arduinoData{:});
        ExpData(iTest).tunnelFile = fullfile(fileRootDir, 'windtunnelData', ...
                                             idTable(iTest, :).windtunnelData{:});
        ExpData(iTest).dx = idTable(iTest, :).dstX;
        ExpData(iTest).airspeed = idTable(iTest, :).airspeed;
        ExpData(iTest).freq = idTable(iTest, :).target_freq;

        fprintf('Processing file: %s\n', ExpData(iTest).tunnelFile);

        % Load, clean and sync experimental data
        tunnelData = loadtunnel(ExpData(iTest).tunnelFile, ...
                                CUTOFF_MULTIPLIER * idTable(iTest, :).target_freq);

        if ~isempty(ExpData(iTest).arduFile) % Not for static tests (no ardu data)
            [arduShort, ~] = loadardu(ExpData(iTest).arduFile);

            % DEBUG
            % figure;
            % hold on;
            % plot(tunnelData(:, 1));
            % plot(tunnelData(:, 3));
            % grid on;

            [arduShort, tunnelData, ExpData(iTest).arduStart, ExpData(iTest).tunnelStart] = ...
                syncdata(arduShort, tunnelData);

            % Save stuff for later use
            ExpData(iTest).time = arduShort(:, 1);
            ExpData(iTest).Front.angles = arduShort(:, 2);
            ExpData(iTest).Front.power = arduShort(:, 4);
            ExpData(iTest).Aft.angles = arduShort(:, 3);
            ExpData(iTest).Aft.power = arduShort(:, 5);

        else
            arduShort = [];
            arduFull = [];

            % Save stuff for later use
            ExpData(iTest).time = 0:1 / 1000:(size(tunnelData, 1) - 1) / 1000; % FIXME: no magic NB
            ExpData(iTest).Front.angles = ones(size(ExpData(iTest).time)) * ...
                idTable(iTest, :).Angles1;
            ExpData(iTest).Aft.angles = ones(size(ExpData(iTest).time)) * ...
                idTable(iTest, :).Angles2;
            ExpData(iTest).Front.power = [];
            ExpData(iTest).Aft.power = [];

        end

        % Better variables
        ExpData(iTest).sampling = 1 / diff(ExpData(iTest).time(1:2));
        ExpData(iTest).Front.forces = tunnelData(:, 1:3);
        ExpData(iTest).Front.moments = tunnelData(:, 4:6);
        ExpData(iTest).Aft.forces = tunnelData(:, 6 + (1:3));
        ExpData(iTest).Aft.moments = tunnelData(:, 6 + (4:6));

        ExpData(iTest).Testcase.dx = idTable(iTest, :).dstX;
        ExpData(iTest).Testcase.airspeed = idTable(iTest, :).airspeed;
        ExpData(iTest).Testcase.freq = idTable(iTest, :).target_freq;
        ExpData(iTest).Testcase.temp = idTable(iTest, :).temperature;
        ExpData(iTest).Testcase.press = idTable(iTest, :).pressure;

        if PLOT_CLEANED
            figure;
            hold on;
            plot(ExpData(iTest).Front.angles);
            yyaxis right;
            plot(ExpData(iTest).Front.forces(:, 3));

        end

        % Save the cleaned data
        savecsv(ExpData(iTest), arduShort, tunnelData);

    end

    % Save ExpData structure to MAT file
    savemat(idFile, ExpData);

end
