function ResData = resultsanalysis(matFiles)
    % RESULTSANALYSIS Analysis of the different results obtained
    %   Todo

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    close all;
    clc;

    addpath(genpath('.'));

    % Defaults and constants
    persistent matFiles_
    persistent ResData_

    PLOT_RES = false;
    FORCE_RELOAD = false;
    OFFSET_VALUES = 0;

    if (isempty(matFiles_) || ~strcmp(matFiles_, matFiles)) && ~FORCE_RELOAD

        % Load data
        allFiles = dir(matFiles);
        for iFile = 1:numel(allFiles)
            resFile = fullfile(allFiles(iFile).folder, allFiles(iFile).name);
            tmp = load(resFile);
            ResData(iFile, :) = tmp.ExpData;
        end
        % Remove useless data
        ResData = rmfield(ResData, {'arduStart', 'tunnelStart'}); % Useless

        % FIXME: remove deprecated
        ResData = rmfield(ResData, {'arduFile', 'tunnelFile', ...
                                    'dx', 'airspeed', 'freq'});

        % Cache values to prevent excessive reloads
        matFiles_ = matFiles;
        ResData_ = ResData;
    else
        ResData = ResData_;
    end

    % Get all correct useful phases
    ResData = calcallphases(ResData);
    ResData = calcinertia(ResData);
    ResData = calctrueforces(ResData);
    ResData = calccoeff(ResData);

    plotcf(ResData);
    % plotclcd(ResData);

end

function ResData = calcallphases(ResData)
    % GETALLPHASES Determine the range for all phase offsets

    for i = 1:size(ResData, 1)
        for j = 1:size(ResData, 2)

            % FIXME Refactor the rest to use this!
            ResData(i, j).Front.trueFreq = getmainfrequency(ResData(i, j).Front.angles, ...
                                                            1 / diff(ResData(i, j).time(1:2)));
            ResData(i, j).Aft.trueFreq = getmainfrequency(ResData(i, j).Aft.angles, ...
                                                          1 / diff(ResData(i, j).time(1:2)));

            fprintf('\nFinding phase for element (%d,%d): %0.2fm, %0.2fHz, %0.1fm/s\n', ...
                    i, j, ...
                    ResData(i, j).Testcase.dx, ...
                    ResData(i, j).Testcase.freq, ...
                    ResData(i, j).Testcase.airspeed);

            % Find all phase offsets between front and aft
            [offsetsVal, signalParts] = findallphasesdiff(ResData(i, j));
            ResData(i, j).AllPhases.offsets = offsetsVal;
            ResData(i, j).AllPhases.parts = signalParts;
        end
    end
end

function ResData = calcinertia(ResData)
    % CALCINERTIA Calcualte inertia forces

    for i = 1:size(ResData, 1)
        for j = 1:size(ResData, 2)

            % Index of the Wind-off data
            woIdx = (j) - mod(j - 1, 4);
            ResDataWO = ResData(i, woIdx);

            for iPos = {'Front', 'Aft'}
                pos = string(iPos);

                % Get first and last peaks for wind-off conditions
                woBounds = getboundingpeaks(ResDataWO.(pos).angles, ResDataWO.sampling);
                woIdx = woBounds(1):woBounds(2);

                % Calculate mean force and moments due to inertia
                meanForces = mean(ResDataWO.(pos).forces(woIdx, :));
                meanMom = mean(ResDataWO.(pos).moments(woIdx, :));

                ResData(i, j).AllPhases.(pos).inertiaF = meanForces;
                ResData(i, j).AllPhases.(pos).inertiaM = meanMom;
            end

        end
    end
end
