function dens = airdensity(temp, press)
    % airdensity Calculates air density from temperature and pressure using perfect gas law.
    % -----
    %
    % Syntax:
    %   dens = airdensity(temp) calculates the air density from the temperature, using the prefect
    %   gas law. Assumes that pressure = 101325 Pa.
    %
    %   dens = airdensity(temp, press) calculates the air density from the temperature and pressure,
    %   using the prefect gas law.
    %
    % Inputs:
    %   temp  : Air temperature, [K]
    %   press : Air pressure, [Pa]
    %
    % Output:
    %   dens  : Air density, [kg/m^3]
    %
    % See also: process_wt.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    GAS_CONST = 287.0500676; % Specific gas constant, [J/(kg K)]
    SL_PRESSURE = 101325;   % Sea Level pressure, [Pa]

    % Input checks
    if nargin < 2
        press = SL_PRESSURE;
    end

    dens = press ./ (GAS_CONST * temp);

end
