function peakRanges = findwingphase(ResData, phaseDeg)
    % FINDWINDPHASE Return the indexes where the wings have a certain phase difference
    %   Todo

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Constants and defautls
    N_PHASE_PEAKS = 5; % Number of peaks in phase to return ideally

    % Abbreviations
    sampling = 1 / diff(ResData.time(1:2));

    % Find flapping frequencies and period from data
    trueFreq(1) = getmainfrequency(ResData.time, ResData.Front.angles);
    trueFreq(2) = getmainfrequency(ResData.time, ResData.Aft.angles);
    period = 1 ./ [trueFreq(1), trueFreq(1)]; % Signal perdio, [s]
    nPeriodIdx = period * sampling; % Number of indexes in a period

    fprintf('    TrueFreq = %0.2f/%0.2fHz, theory = %0.2fHz\n', ...
            trueFreq(1), trueFreq(2), ResData.Testcase.freq);

    % Determine the ranges when the two angles have the proper phase difference
    peakRanges = findsyncedpeaks(ResData.Front.angles, ...
                                 ResData.Aft.angles, ...
                                 nPeriodIdx(1), phaseDeg, N_PHASE_PEAKS);

end

function trueFreq = getmainfrequency(time, signal)
    % GETFREQUENCIES Return the main signal frequencies

    % Constants and defaults
    DEBUG = false;

    % Signal parameters
    sampling = 1 / abs(diff(time(1:2)));
    if mod(length(signal), 2) ~= 0
        signal = signal(1:end - 1);
    end
    nts = length(signal);

    % Fourier transform
    transform = fft(signal);

    fT2 = abs(transform / nts); % Two-sided spectrum
    fT1 = fT2(1:nts / 2 + 1); % Single-sided spectrum
    fT1(2:end - 1) = 2 * fT1(2:end - 1);
    freq = sampling * (0:(nts / 2)) / nts; % Frequency domain

    % Get main frequency
    [~, trueFreqIdx] = max(fT1(2:end));
    trueFreq = freq(1 + trueFreqIdx);

    if DEBUG
        figure;
        plot(time, signal);
        grid on;

        figure;
        plot(freq, fT1);
        title("Single-Sided Amplitude Spectrum of S(t)");
        xlabel("f (Hz)");
        ylabel("|P1(f)|");
        grid on;
    end

end

function peakLocs = findallpeaks(signal, nPeriodIdx)
    % FINDANGLEPEAKS Find all true peaks in the signal

    maxAngle = max(signal);

    [~, peakLocs] = findpeaks(signal, ...
                              'MinPeakProminence', maxAngle / 2, ...
                              'MinPeakDistance', nPeriodIdx / 2);
end

function signalRange = findsyncedpeaks(signal1, signal2, nPeriodIdx, offset, nPeaks)
    % FINDANGLEPEAKS Find peaks in the signals that are synced with respect to the offset desired
    %   Returns signalRange : index of peak before and peak after we are sync

    % Constants and defaults
    DEBUG = false;
    OFFSET_TOL = 2e-2;
    DISABLE_WARNING = true;

    % Find peaks in the wing angles
    peakLocs1 = findallpeaks(signal1, nPeriodIdx);
    peakLocs2 = findallpeaks(signal2, nPeriodIdx);
    npeaks = min(length(peakLocs1), length(peakLocs2));

    % Trim to same number of peaks
    peakLocs1 = peakLocs1(1:npeaks);
    peakLocs2 = peakLocs2(1:npeaks);

    % Peaks where offset is good
    offsets = getphaseoffsets(peakLocs1, peakLocs2, nPeriodIdx);
    [offVal, offsetIdx] = mink(abs(offsets - offset), nPeaks);
    % Remove if we start or end by a peak
    limitsIdx = ismember(offsetIdx, [1, length(peakLocs1)]);
    offVal(limitsIdx) = [];
    offsetIdx(limitsIdx) = [];

    if any(offVal > OFFSET_TOL * 360)
        offsetIdx = offsetIdx(offVal <= OFFSET_TOL * 360);
        if ~DISABLE_WARNING
            warning('findwingphase:findsyncedpeaks:wrongPhaseOffset', ...
                    ['Some peaks have a phase difference larger than %0.2f° from what was '...
                     'requested. '...
                     'Reducing the number of peaks to match the tolerance criterion.\n'...
                     'New number of peaks considered = %d.\n'], OFFSET_TOL * 360, ...
                    length(offsetIdx));
        end
        if isempty(offsetIdx)
            fprintf(2, '      Phase difference of %0.2f not found!\n', offset);
        end

    end

    % Signal ranges where it is properly synchronized
    signalRange = peakLocs1([offsetIdx - 1, offsetIdx + 1]);

    % FIXME: Hackyway to make it work
    if numel(signalRange) == 2
        signalRange = signalRange';
    end

    if DEBUG
        peakLocs = peakLocs1(offsetIdx); % Location of the peaks
        figure;
        hold on;
        plot(signal1);
        plot(signal2);
        for i = 1:length(peakLocs)
            plot([peakLocs(i), peakLocs(i)], ylim, '-k', 'Linewidth', 1.2);
        end
        hold off;
        grid on;
        legend('Front', 'Aft');
        pause;
    end

end

function offsets = getphaseoffsets(peaks1, peaks2, nPeriodIdx)
    % GETPHASEOFFSETS Calculate phase offsets between every contiguous peaks
    %   Note: Offset is positive when peak2 is AFTER peak1

    offsets = peaks2 - peaks1;

    offsets = offsets * 360 / nPeriodIdx;

end
