function ParsedDat = plotstatictests(StatData)
    % PLOTSTATICTESTS Plot CL and CD for each set of wing w.r.t front and aft dihedral
    %   Todo

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    close all;

    % Constants and defaults
    CHORD = 0.05;
    SPAN = 2 * 0.2;
    C_TO_K = 273.15;
    AIRSPEEDS = [5, 8, 10, 14];

    % Create useful vectors for each sets of front/aft position
    % Vectors dimensions = number of speeds

    % First we get all unique angles for front and aft wings that were tested
    for i = numel(StatData):-1:1
        angleF(i) = StatData(i).Front.angles(1);
        angleA(i) = StatData(i).Aft.angles(1);
    end
    angleF = unique(angleF);
    angleA = unique(angleA);
    nVeloc = numel(StatData) / (length(angleF) * length(angleA)); % Number of velocities per config

    % Create base mesh grid for all tests cases
    [gridF, gridA] = meshgrid(angleF, angleA);

    % Initialize force coefficients in X and Y for Front and Aft
    cfzF = zeros([size(gridF), nVeloc]);
    cfxF = zeros([size(gridF), nVeloc]);
    cfzA = zeros([size(gridF), nVeloc]);
    cfxA = zeros([size(gridF), nVeloc]);
    stdfz = zeros([size(gridF), nVeloc]);

    for i = 1:numel(StatData)

        idx1 = gridF == StatData(i).Front.angles(1) & gridA == StatData(i).Aft.angles(1);
        [~, idx2] = min(abs(AIRSPEEDS - StatData(i).Testcase.airspeed));

        for iPos = {'Front', 'Aft'}
            pos = string(iPos);

            fz = mean(StatData(i).(pos).forces(:, 3));
            fx = mean(StatData(i).(pos).forces(:, 1));
            %             stdfz = std(StatData(i).(pos).forces(:, 3));
            %             stdfx = std(StatData(i).(pos).forces(:, 1));

            dens = airdensity(StatData(i).Testcase.temp + C_TO_K, ...
                              StatData(i).Testcase.press);
            speed = StatData(i).Testcase.airspeed;

            cfz = forcetocoeff(fz, dens, speed, CHORD, SPAN, 'force');
            cfx = forcetocoeff(fx, dens, speed, CHORD, SPAN, 'force');

            % Nondim with projected area
            cfz = cfz / sind(90 + StatData(i).(pos).angles(1));
            cfx = cfx / sind(90 + StatData(i).(pos).angles(1));

            if strcmp(pos, 'Front')
                cfzF(:, :, idx2) = cfzF(:, :, idx2) + idx1 * fz;
                cfxF(:, :, idx2) = cfxF(:, :, idx2) + idx1 * fx;
                cfzF(:, :, idx2) = cfzF(:, :, idx2) + idx1 * fz;
                cfxF(:, :, idx2) = cfxF(:, :, idx2) + idx1 * fx;
            else
                cfzA(:, :, idx2) = cfzA(:, :, idx2) + idx1 * fz;
                cfxA(:, :, idx2) = cfxA(:, :, idx2) + idx1 * fx;
            end

        end

    end

    ParsedDat.cfzF = cfzF;
    ParsedDat.cfxF = cfxF;
    ParsedDat.cfzA = cfzA;
    ParsedDat.cfxA = cfxA;

    for i = 1:nVeloc
        figure('Name', ['V=', i]);
        ax1 = subplot(221); % Front CL
        surf(gridF, gridA, cfzF(:, :, i));
        setgca();

        ax2 = subplot(222); % Front CD
        surf(gridF, gridA, cfxF(:, :, i));
        setgca();

        ax3 = subplot(223); % Aft CL
        surf(gridF, gridA, cfzA(:, :, i));
        setgca();

        ax4 = subplot(224); % Aft CD
        surf(gridF, gridA, cfxA(:, :, i));
        setgca();

        %         linkprop([ax1 ax3], 'Zlim'); % CL plots
        %         linkprop([ax2 ax4], 'Zlim'); % CD plots
    end

    for i = 1:length(angleF)
        figure;
        size(angleA);
        size(cfzF(i, :, 4));
        hold on;
        title([angleF(i)]);
        errobar(angleA, cfzF(:, i, 3), stdfzF(:, i, 3), '-o');
        plot(angleA, cfzF(:, i, 3), '-o');
        plot(angleA, cfzA(:, i, 3), '-o');
        plot(angleA, cfzF(:, i, 3) + cfzA(:, i, 3), '--x');
        hold on;
    end

end

function setgca()
    % SETGCA Sets the axes parameters

    xlabel('Front');
    ylabel('Aft');
    grid on;
end
