function [arduShort, arduData] = loadardu(arduFile)
    % LOADARDU Load the Arduino data and clean it.
    %   This function loads the Arduino data. Then it:
    %       - removes the unneeded columns
    %       - convert time into milliseconds
    %       - converts the output into an array instead of a table
    %       - calculates the power
    %       - re-aligns the angles properly from the raw wing positions
    % -----
    % Output format:
    %   The output array has one row per timestep. The columns are ordered as follows:
    %       time, angleFrontLeft, angleFrontRight, angleAftLeft, angleAftRight, powerFront, powerAft
    %
    % Syntax:
    %   arduData = loadardu(arduFile) load the Arduino data file, then clean it and return a cleaned
    %   version with only the relevant fields in form of an array.
    %
    % Inputs:
    %   arduFile: Arduino file.
    %
    % Outputs:
    %   arduData: Array with the cleaned data.
    %
    % See also: readdata.

    % ----------------------------------------------------------------------------------------------
    % TODO: Check for inconsistencies in the table
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Load data
    arduTable = table;
    if ~isempty(arduFile)
        arduTable = loadcsv([arduFile, '.csv']);
    end

    arduTable.time = timetosec(arduTable.time);

    % Remove data with duplicate timecode
    [~, idx] = unique(arduTable.time); % indexes of unique timecodes

    arduTable = arduTable(idx, :);

    % Convert into table
    arduRaw = table2array(arduTable);
    time = arduRaw(:, 1);

    % Calculate power, remove tension and current
    power = arduRaw(:, [6, 8]) .* arduRaw(:, [7, 9]);

    % Recalculate angles properly from wing positions recoreded
    goodAngles = fixangles(arduRaw(:, 2:5));

    % Take mean angles for each set of wings (diff is almost negligible)
    meanAnglesFront = mean(goodAngles(:, 1:2), 2);
    meanAnglesAft = mean(goodAngles(:, 3:4), 2);

    % Output a cleaned up tables
    arduData = [time, goodAngles, power]; % time, FL, FR, AL, AR, PowF, PowA
    arduShort = [time, meanAnglesFront, meanAnglesAft, power];
end

function rawTable = loadcsv(file)
    % LOADCSV Load the csv into a table

    % Tweak options before import
    opts = detectImportOptions(file);
    opts.VariableNamesLine = 1;

    opts.SelectedVariableNames = {'x_RXDate_Time', 'x1_Left_field6_', 'x1_Right_field7_', ...
                                  'x2_Left_field8_', 'x2_Right_field9_', 'x1_Tension_field14_', ...
                                  'x1_Current_field15_', 'x2_Tension_field16_', ...
                                  'x2_Current_field17_'};
    opts.DataLines = [2 Inf];

    % Import
    rawTable = readtable(file, opts);

    % Rename columns
    rawTable.Properties.VariableNames = {'time', 'angleFrontLeft', 'angleFrontRight', ...
                                         'angleAftLeft', 'angleAftRight', ...
                                         'tensionF', 'currentF', 'tensionA', 'currentA'};
end

function timeSec = timetosec(time)
    % TIMETOMILLI Converts datetime data into milliseconds since start

    % Parse time properly
    time = datetime(time, 'InputFormat', 'uuuu/MM/dd''/ ''HH:mm:ss::SSS', 'TimeZone', 'UTC');

    % Converts into difference from the first time
    timeMilli = milliseconds(time - time(1)) + 1; % Shift of 1 so first time is 1 millisecond

    timeSec = timeMilli / 1000;
end

function angles = fixangles(wingPos)
    % FIXANGLES Fix the angle values to avoid issues with the 12 bit overflow
    %   Note: 3rd sensor had an encoding issue. This issue is fixed here automatically
    %   Note: Aft sensors enocde in the wrong direction (Angles should be opposite)
    %   Note: Sometimes error with front left wing stuck at 0 or 4095 when not flapping. If stuck,
    %   just copy right wing

    % Defaults and constants
    ENC_PRECISION = 12; % 12-bits encoders
    MAX_WING_ANGLE = 38.07; % Determined by geometry of the setup
    CALIB_OFFSET = [100.8984  115.7520  211.4648  278.6133]; % Maximum angle recored BEFORE offset
    N_VAL_STATIC = 10; % Number of different positions under which it wing is supposed static

    % Abbreviations
    nBits = 2^ENC_PRECISION;
    nBits16 = 2^16;

    % Fix issue with third sensor (aft left wing) where it was converted in a 16-bits int
    % [FIXME] Once this is fixed in the controller, add a switch to correct or not the data
    wingPos(:, 3) = mod(wingPos(:, 3) + nBits / 2, nBits16);

    % Aft sensors are in the wrong direction
    % [FIXME] Once this is fixed in the controller, add a switch to correct or not the data
    wingPos(:, 3:4) = nBits - wingPos(:, 3:4);

    % Convert position data into actual degree
    angles = wingPos * 360 / nBits;

    % maxangles = max(angles)
    % Calibrate values so max angle recored matches MAX_WING_ANGLE
    angles = angles - CALIB_OFFSET + MAX_WING_ANGLE;

    % Fix for front wing stuck in 0 or 4095
    % For some reason the first wing get sometimes stuck in strange values when there is no motion.
    % It then returns a position oscillating between 0 and 4095 (or a bit more than that). This can
    % be detected by simply looking if some diff in position are greater than nBist/2. If it is the
    % case, either we have an abrupt change of +180° (impossible), or there is an overflow issue.

    if (any(diff(wingPos(:, 1)) > nBits / 2) || ...
        all(wingPos(:, 1) == nBits - 1) || ...
        all(wingPos(:, 1) == 0)) && ...
        ~any(diff(wingPos(:, 2)) > nBits / 2)
        angles(:, 1) = angles(:, 2);
    end

end
