function tunnelData = loadtunnel(tunnelFile, cutoff)
    % LOADTUNNEL Load the Wind Tunnel data and clean it.
    %   This function loads the WindTunnel data. Then it:
    %       - applies a low-pass filter to clean the signal
    % -----
    % Output format:
    %   The output array has one row per timestep. The columns are ordered as follows:
    %       time, fxF, fyF, fzF, mxF, myF, mzF, fxA, fyA, fzA, mxA, myA, mzA
    %
    % Syntax:
    %   tunnelData = loadtunnel(tunnelFile) load the wind tunnel data file, clean it and return a
    %   cleaned version with only the relevant portion in form of an array.
    %
    % Inputs:
    %   tunnelFile : Wind tunnel file
    %   cutoff     : Cutoff frequency for the low-pass filter, [Hz]
    %
    % Outputs:
    %   tunnelData: Array with the cleaned data.
    %
    % See also: readdata.

    % ----------------------------------------------------------------------------------------------
    % TODO: Check for inconsistencies in the table
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    SAMPLING = 1000; % Sampling rate, [Hz]

    % Load data
    tunnelTable = table;
    if ~isempty(tunnelFile)
        tunnelTable = loadcsv([tunnelFile, '.txt']);
    end

    % Convert into table
    raw = table2array(tunnelTable);

    % Filter flapping signal, not needed in static
    if cutoff ~= 0
        filtered = filterwt(raw, cutoff, SAMPLING);
    else
        filtered = raw;
    end

    % Calibrate
    calibrated = calibrate(filtered);

    % Return cleaned data
    tunnelData = calibrated;
end

function rawTable = loadcsv(file)
    % LOADCSV Load the csv into a table

    % Tweak options before import
    opts = detectImportOptions(file);
    opts.VariableNames = {'Fx_front', 'Fy_front', 'Fz_front', ...
                          'Mx_front', 'My_front', 'Mz_front', ...
                          'Fx_aft', 'Fy_aft', 'Fz_aft', ...
                          'Mx_aft', 'My_aft', 'Mz_aft'};
    opts.DataLines = [1 Inf];

    % Import
    rawTable = readtable(file, opts);
end

function calibrated = calibrate(raw)
    % CALIBRATE Calibrate a force or a moment properly
    %   My creates a parasitic force Fx that must be substracted from the real Fx

    persistent kF kA % Calibration factors for My-Fx parasitic force

    calibrated = raw;

    % Proceed to calibration using the calibration data
    if isempty(kF) || isempty(kA)
        kF = findcalibslope('data/wtFiles/2022/calibration/calibFront_*', 'Fx', 'My');
        kA = findcalibslope('data/wtFiles/2022/calibration/calibAft_*', 'Fx', 'My');
    end

    % Calibrate Fx by removing parasitic force due to My
    calibrated(:, [1, 1 + 6]) = calibrated(:, [1, 1 + 6]) - [kF kA] .* calibrated(:, [5, 5 + 6]);
end

function filtered = filterwt(raw, cutoff, sampling)
    % FILTERWT Apply a low-pass filter to the raw data

    % Design a butterworth IIR filter
    nyq = sampling / 2;
    [bb, aa] = butter(3, cutoff / nyq, 'low');

    % Apply filter
    filtered = filtfilt(bb, aa, raw(:, :));
end

function slope = findcalibslope(files, force, moment)
    % FINDCALIBSLOPE Find the linear coefficient for the calibration of parasitic effects

    % Constants and defaults
    FORCES = {'Fx', 'Fy', 'Fz'};  % Order of forces in datafile
    MOMENTS = {'Mx', 'My', 'Mz'}; % Order of moments in datafile

    % Get all relevant calibration files
    allFiles = dir(files);

    iForce = find(strcmp(force, FORCES));
    iMom = find(strcmp(moment, MOMENTS));

    % Get mean forces and moment for each calibration point
    meanF = zeros(1, length(allFiles));
    meanM = zeros(1, length(allFiles));
    for iFile = 1:length(allFiles)

        % Determine if we must look at front or aft data columns
        if contains(allFiles(iFile).name, 'calibFront')
            idx = 1;
        elseif contains(allFiles(iFile).name, 'calibAft')
            idx = 2;
        else
            error('loadtunnel:findcalibslope:undefinedCalibType', ...
                  'Calibration filename must contain either ''calibFront'' or ''calibAft''.');
        end

        % Load raw forces and moments
        calibTable = loadcsv(fullfile(allFiles(iFile).folder, allFiles(iFile).name));
        rawCalib = table2array(calibTable);

        % Isolate proper data
        rawF = rawCalib(:, 6 * (idx - 1) + iForce);
        rawM = rawCalib(:, 6 * (idx - 1) + (3 + iMom));

        meanF(iFile) = mean(rawF);
        meanM(iFile) = mean(rawM);

    end

    % Slope is mean value of the individual slope between each calibration point
    slope = mean(diff(meanF) ./ diff(meanM));

end
