function plottime(ExpData, arduType, plotStart)
    % PLOTTIME Plot experimental data evolution over time.
    %   This function produces the following plots for each module:
    %       - angle left, angle right, Fz vs time
    %       - angle left, angle right, Fx vs time
    % -----
    %
    % Syntax:
    %   plottime(ExpData) plots relevant experimental data versus time.
    %
    % Inputs:
    %   ExpData : Structure with the experimental data
    %
    % Outputs:
    %   Various plots
    %
    % See also: processall.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Constants and defaults
    SAMPLING = 1000;
    INTERVAL = 1:5000; %  Plot only 5 seconds

    % ALL_FORCES = {'Fx', 'Fy', 'Fz', 'Mx', 'My', 'Mz'};
    %     ALL_FORCES = {'Fx', 'Fz'};
    ALL_FORCES = {'Fz'};

    % Plot everything useful
    for iDat = 1:numel(ExpData)
        expParam = sprintf('d = %0.2f, f = %0.2f, V = %0.1f', ...
                           ExpData(iDat).dx, ExpData(iDat).freq, ExpData(iDat).airspeed);
        for iForce = 1:numel(ALL_FORCES)
            force = ALL_FORCES{iForce};
            figure('Name', force);
            title(ExpData(iDat).tunnelFile);
            subplot(211);
            title(['Front module -- ', expParam]);
            plotwithangle(ExpData(iDat), arduType, 'front', force, SAMPLING, INTERVAL, plotStart);
            subplot(212);
            title(['Aft module -- ', expParam]);
            plotwithangle(ExpData(iDat), arduType, 'aft', force, SAMPLING, INTERVAL, plotStart);
        end
    end

end

function plotwithangle(ExpData, arduType, position, force, sampling, interval, plotStart)
    % PLOTWITHANGLE Plot the 'force' measured for the module in 'position' alongside the wing angles

    WT_DATA_TYPES = {'Fx', 'Fy', 'Fz', 'Mx', 'My', 'Mz'};
    wtTime = 0:1 / sampling:(size(ExpData.(position).tunnelData, 1) - 1) / sampling;

    arduStart = ExpData.arduStart;
    tunnelStart = ExpData.tunnelStart;
    % Plot the wing angles
    yyaxis left;
    hold on;
    if strcmp(arduType, 'arduShort')
        idx = 2;
    else
        idx = 2:3;
    end
    for i = idx
        plot(ExpData.(position).(arduType)(interval, 1), ...
             ExpData.(position).(arduType)(interval, i));
        if plotStart
            plot(arduStart, ...
                 ExpData.(position).(arduType)(arduStart * sampling, i), 'ok');
        end
    end

    % Plot the wind tunnel data
    yyaxis right;
    plot(wtTime(interval), ...
         ExpData.(position).tunnelData(interval, strcmp(WT_DATA_TYPES, force)));
    if plotStart
        plot(tunnelStart, ...
             ExpData.(position).tunnelData(round((tunnelStart) * sampling), ...
                                           strcmp(WT_DATA_TYPES, force)), ...
             'sk');
    end
    hold off;
    grid on;

    if strcmp(position, 'aft')
        if strcmp(arduType, 'arduShort')
            legend('Location', 'bestoutside', 'orientation', 'horizontal', 'Wings angle', force);

        else
            legend('Location', 'bestoutside', 'orientation', 'horizontal', 'angleL', 'angleR', ...
                   force);
        end
    end

end
