function [arduSynced, tunnelSynced, arduStartTime, tunnelStartTime] = ...
        syncdata(arduShort, tunnelData)
    % SYNCDATA Synchronize datasets from the Arduio and the Wind Tunnel.
    % -----
    %   The acquisition of data is realized from two different sources on two different machines:
    %       - load sensors (dataTunnel)
    %       - wing position (dataArdu)
    %   This functions ensures both data are synchronized by looking at the start of the flapping
    %   motion.
    %
    % Syntax:
    %   [syncArdu, syncTunnel] = syncdata(dataArdu, dataTunnel) returns Arduino and wind tunnel data
    %   arrays in sync and interpolated over the same range.
    %
    % Inputs:
    %   dataArdu   : Arduino data [FIXME]
    %   dataTunnel : Wind tunnel data [FIXME]
    %
    % Output:
    %   syncArdu   : Arduino data synced [FIXME]
    %   syncTunnel : Wind tunnel data synced[FIXME]
    %
    % See also: process_wt.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    WT_SAMPLING = 1000; % Sampling rate for the wind tunnel data, [hZ]
    START_TRIM_SECONDS = 10; % Number of seconds to trim at the beginning (motor speed up), [s]
    END_TRIM_SECONDS = 2; % Number of seconds to trim at the end, [s]

    % Detect start of wing motion in Arduino and wind tunnel datasets
    [arduStartIdx, arduStartTime, steadyIdx] = getstartardu(arduShort);
    [tunnelStartIdx, tunnelStartTime] = getstarttunnel(tunnelData, steadyIdx);

    % Trim everything before wing starts in both datasets
    arduShort = arduShort(arduStartIdx:end, :);
    tunnelData = tunnelData(tunnelStartIdx:end, :);

    % Set both datasets time to 0 for the first element
    arduShort(:, 1) = arduShort(:, 1) - arduShort(1, 1);

    % Interpolate arduino over wt time sampling
    time = 0:1 / WT_SAMPLING:size(tunnelData, 1) / WT_SAMPLING;
    dummy = interp1(arduShort(:, 1), arduShort(:, 2:end), time);
    arduShort = [time', dummy];
    nanIdx = find(isnan(arduShort(:, 2)));
    arduShort(nanIdx:end, :) = [];     % Remove extrapolated values

    % Cut longests dataset
    maxTime = min(size(arduShort, 1), size(tunnelData, 1));
    arduShort(maxTime:end, :) = [];
    tunnelData(maxTime:end, :) = [];

    % Assign output properly now that we are synced
    arduSynced = arduShort;
    tunnelSynced = tunnelData;

    % Trim first and lasts seconds of datasets (remove possible transient effects)
    arduSynced = arduSynced(START_TRIM_SECONDS * WT_SAMPLING:end, :);
    arduSynced(:, 1) = arduSynced(:, 1)  - arduSynced(1, 1); % Make it start at zero again
    tunnelSynced = tunnelSynced(START_TRIM_SECONDS * WT_SAMPLING:end, :);
    arduSynced(end - END_TRIM_SECONDS * WT_SAMPLING:end, :) = [];
    tunnelSynced(end - END_TRIM_SECONDS * WT_SAMPLING:end, :) = [];

end

function [startIdx, startTime, steadyIdx] = getstartardu(arduData)
    % GETSTARTARDU Get the starting po  int for the arduino dataset and the flapping direction at
    % start

    ANGLE_IDX = [2, 3]; % Indexes of the front and aft angles in arduData file
    N_STABLE_VAL = 20; % Number of stable values before beginnig of motion
    DCRIT = 0.1; % Deviation from the mean to be considered a "large" deviation
    STD_NO_MOTION = 3; % A standard deviation less than 1deg means no motion

    % Get a first approximation for the starting point of each wing set
    startIdx = [NaN, NaN];
    steadyIdx = [NaN, NaN];
    for i = 1:2
        if std(arduData(:, ANGLE_IDX(i))) > STD_NO_MOTION
            startIdx(i) = findsignalstart(arduData(:, ANGLE_IDX(i)), N_STABLE_VAL, 'ardu', DCRIT);
        else
            steadyIdx(i) = i;
        end
    end
    startIdx = min(startIdx);
    startTime = arduData(startIdx, 1);
end

function [startIdx, startTime] = getstarttunnel(tunnelData, steadyIdx)
    % GETSTARTTUNNEL Get the starting point for the wind tunnel dataset

    % Only use the Fz to determine starting point
    FZ_IDX = [3, 9]; % Fz indexes in tunnelData for front and aft set
    SAMPLING = 1000; % Data sampling rate
    TRIM_MS = 500; % Trim some MS at beginning to avoid noise
    DCRIT = 0.1; % Deviation from the mean to be considered a "large" deviation

    % Start by trimming the first milliseconds to avoid starting noise
    tunnelData = tunnelData(TRIM_MS:end, :);

    startIdx = [NaN, NaN];
    for i = 1:2
        if steadyIdx(i) ~= i
            startIdx(i) = findsignalstart(tunnelData(:, FZ_IDX(i)), SAMPLING / 2, 'tunnel', DCRIT);
        end
    end
    startIdx = TRIM_MS + min(startIdx);
    startTime = startIdx / SAMPLING;

end

function startIdx = findsignalstart(signal, stableIdx, type, fixCrit)
    % FINDSIGNALSTART Find start of motion by looking at deviations from a 'stable' signal

    % Constants and defaults
    LARGE_DEV_STD = 10; % Multiplier of std to be considered 'large deviation'
    DEBUG = false;

    if nargin < 4
        fixCrit = 0;
    end

    % Analysis of stable portion of the signal
    meanStable = mean(signal(1:stableIdx));
    stdStable = std(signal(1:stableIdx));

    % Find first large deviation from the mean
    devCrit = max(fixCrit, LARGE_DEV_STD * stdStable);
    meanDev = (abs(signal - meanStable));
    largeDevIdx = find(meanDev > devCrit);

    % Start point is the last extremum before first large deviation
    dSignal = diff(signal(1:largeDevIdx(1)), 1);
    inflexPts = find([0; diff(sign(dSignal))] ~= 0); % Zeros to get idx in signal and not dSignal
    startIdx = inflexPts(end);

    % For WT, the start is not the last extremum, but the last minimum in the derivate
    if strcmp(type, 'tunnel')
        [~, newIdx] = min(meanDev(startIdx:largeDevIdx(1)));
        startIdx = (startIdx - 1) + newIdx;
    end

    if DEBUG
        cutoff = ones(size(signal)) * devCrit;
        figure;
        hold on;
        plot(signal);
        plot(meanDev);
        plot(dSignal);
        plot(cutoff, '--k');
        plot([largeDevIdx(1), largeDevIdx(1)], ylim, '--k');
        plot([startIdx, startIdx], ylim, '--b');
        legend('Signal', 'Deviation from mean', 'dSignal');
        grid on;
        xlim([0 4000]);
        pause;
    end

end
